unit Histograph;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  extCtrls;

const
  MAXVALUES = 300;

type
  PInt = ^Integer;
  THistograph = class(TGraphicControl)
  private
    FBackgroundColor, FGridColor,
    FHighColor, FNormalColor, FLowColor: TColor;
    FMaxValue, FHighValue, FLowValue, FMinValue, FValue: integer;
    FActive, FTransparent: boolean;
    FNumValues: integer;
    FInterval, FIntervalOld: integer;
    FHorizLinesInterval, FVertLinesInterval: integer;
    FTimer: TTimer;
    FValuesBMP, FValues2BMP: TBitmap;
    FGridBMP: TBitmap;
    FAuxBMP: TBitMap;
    FValues: array[1..MAXVALUES] of integer;
    FOldWidth, FOldHeight: integer;

    procedure setBackgroundColor (newValue: TColor);
    procedure setGridColor (newValue: TColor);
    procedure setHighColor (newValue: TColor);
    procedure setNormalColor (newValue: TColor);
    procedure setLowColor (newValue: TColor);
    procedure setMaxValue (newValue: integer);
    procedure setHighValue (newValue: integer);
    procedure setLowValue (newValue: integer);
    procedure setMinValue (newValue: integer);
    procedure setValue (newValue: integer);
    procedure setActive (newValue: boolean);
    procedure setTransparent (newValue: boolean);
    procedure setInterval (newValue: integer);
    procedure setHorizLinesInterval (newValue: integer);
    procedure setVertLinesInterval (newValue: integer);

    procedure repaintGridBMP;
    procedure repaintValuesBMP;
    procedure updateValuesBMP;
    procedure adjustTimer;
    procedure OnTimer (sender: TObject);
    function colorPluma (value: integer): TColor;

  protected
    procedure paint; override;

  public
    constructor Create (AOwner: TComponent); override;
    destructor Destroy; override;
    procedure clear;

  published
    property Align default alNone;
    property Width default 200;
    property Height default 100;

    property BackgroundColor: TColor read FBackGroundColor write setBackgroundColor default clBlack;
    property GridColor: TColor read FGridColor write setGridColor default clSilver;
    property GraphHighColor: TColor read FHighColor write setHighColor default clRed;
    property GraphNormalColor: TColor read FNormalColor write setNormalColor default clLime;
    property GraphLowColor: TColor read FLowColor write setLowColor default clYellow;
    property MaxValue: integer read FMaxValue write setMaxValue default 100;
    property HighValue: integer read FHighValue write setHighValue default 80;
    property LowValue: integer read FLowValue write setLowValue default 20;
    property MinValue: integer read FMinValue write setMinValue default 0;
    property Value: integer read FValue write setValue default 50;
    property Active: boolean read FActive write setActive default true;
    property Transparent: boolean read FTransparent write setTransparent default true;
    property Interval: integer read FInterval write setInterval default 2;
    property HorizLinesInterval: integer read FHorizlinesInterval write setHorizLinesInterval default 10;
    property VertLinesInterval: integer read FVertLinesInterval write setVertLinesInterval default 60;
  end;

procedure Register;

implementation

constructor THistograph.Create (AOwner: TComponent);
begin
inherited Create (AOwner);
controlStyle := [csOpaque];

width := 200;
Height := 100;

FBackgroundColor := clBlack;
FGridColor := clSilver;
FHighColor := clRed;
FNormalColor := clLime;
FLowColor := clYellow;
FTransparent := true;

FMaxValue := 100;
FHighValue := 80;
FLowValue := 20;
FMinValue := 0;
FValue := 50;

FActive := true;
FNumValues := 100;
FInterval := 2;
FHorizLinesInterval := 10;
FVertLinesInterval := 60;

FTimer := TTimer.create (self);
   adjustTimer;
FGridBMP := TBitmap.Create;
FAuxBMP := TBitmap.Create;
FValuesBMP := TBitmap.create;
FValues2BMP := TBitmap.create;

clear;
//for i1 := 1 to MAXVALUES do FValues[i1] := i1;
repaintGridBMP;
repaintValuesBMP;
end;

destructor THistograph.Destroy;
begin
FValuesBMP.free;
FGridBMP.free;
FAuxBMP.free;
FValues2BMP.free;
FTimer.free;

inherited destroy;
end;

procedure THistograph.clear;
var
   i1: integer;
begin
for i1 := 1 to MAXVALUES do
    FValues[i1] := FMinValue;
repaintValuesBMP;
invalidate;
end;

function THistograph.colorPluma (value: integer): TColor;
begin
if value <= FLowValue then
   result := FLowColor
else
   if value >= FHighValue then
      result := FHighColor
   else
      result := FNormalColor;
end;

procedure THistograph.adjustTimer;
begin
FTimer.Interval := self.FInterval * 1000;
FTimer.Enabled := self.FActive;
FTimer.OnTimer := self.OnTimer;
end;

procedure THistograph.setValue (newValue: integer);
begin
FValue := NewValue;
invalidate;
end;

procedure THistograph.setBackgroundColor (newValue: TColor);
begin
FBackgroundColor := NewValue;
repaintGridBMP;
repaintValuesBMP;
invalidate;
end;

procedure THistograph.setGridColor (newValue: TColor);
begin
FGridColor := NewValue;
repaintGridBMP;
invalidate;
end;

procedure THistograph.setHighColor (newValue: TColor);
begin
FHighColor := NewValue;
repaintValuesBMP;
invalidate;
end;

procedure THistograph.setNormalColor (newValue: TColor);
begin
FNormalColor := NewValue;
repaintValuesBMP;
invalidate;
end;

procedure THistograph.setLowColor (newValue: TColor);
begin
FLowColor := NewValue;
repaintValuesBMP;
invalidate;
end;

procedure THistograph.setMaxValue (newValue: integer);
begin
if newValue < FMinValue then exit;
FMaxValue := NewValue;
repaintGridBMP;
repaintValuesBMP;
invalidate;
end;

procedure THistograph.setMinValue (newValue: integer);
begin
if newValue > FMaxValue then exit;
FMinValue := NewValue;
repaintGridBMP;
repaintValuesBMP;
invalidate;
end;

procedure THistograph.setHighValue (newValue: integer);
begin
FHighValue := NewValue;
repaintValuesBMP;
invalidate;
end;

procedure THistograph.setLowValue (newValue: integer);
begin
FLowValue := NewValue;
repaintValuesBMP;
invalidate;
end;

procedure THistograph.setActive (newValue: boolean);
begin
FActive := NewValue;
FTimer.enabled := FActive;
end;

procedure THistograph.setTransparent (newValue: boolean);
begin
FTransparent := NewValue;
repaintValuesBMP;
invalidate;
end;

procedure THistograph.setInterval (newValue: integer);
begin
if newValue < 1 then exit; // Min interval = 2 sec.
FIntervalOld := FInterval; // FIntervalOld needed when repainting values
FInterval := newValue;
FTimer.interval := FInterval * 1000;
repaintGridBMP;
repaintValuesBMP;
end;

procedure THistograph.setHorizLinesInterval (newValue: integer);
begin
FHorizLinesInterval := newValue;
repaintGridBMP;
invalidate;
end;

procedure THistograph.setVertLinesInterval (newValue: integer);
var
   i1: integer;
begin
if newValue < FInterval then exit;
if newValue / FInterval <> 0 then begin// It must be multiple of Finterval begin
   i1 := newValue div FInterval;
   newValue := FInterval * I1; // Finding the multiple lower than newValue
   end;
FVertLinesInterval := newValue;
repaintGridBMP;
invalidate;
end;

procedure THistograph.repaintGridBMP;
var
   wGap, x, y, i1: integer;
   wCoef: double;
begin
with FAuxBMP do begin
     Width := self.clientWidth;
     height := self.clientHeight;
     canvas.brush.color := FBackgroundColor;
     canvas.fillRect (clientRect);
     end;
with FGridBMP do begin
     Width := self.clientWidth;
     height := self.clientHeight;
     canvas.brush.color := FBackgroundColor;
     canvas.fillRect (clientRect);
     canvas.pen.color := FGridColor;
     canvas.pen.width := 1;
     // VerticalLines
     wGap := 2 * (FVertLinesInterval div FInterval);
     x := width - wGap;
     while x >= 0 do begin
           canvas.moveTo (x, 0); canvas.lineTo (x, height);
           dec (x, wGap);
           end;
     // HorizLines
     wCoef := height / (FMaxValue - FMinValue);
     i1 := 1;
     y := round(height - (FHorizLinesInterval * wCoef));
     while y >= 0 do begin
           canvas.moveTo (0, y); canvas.lineTo(width, y);
           inc (i1);
           y := round (height - (i1 * (FHorizLinesInterval * wCoef)));
           end;

     end;

end;

procedure THistograph.repaintValuesBMP;
var
   i1: integer;
   x, y: integer;
   wCoef: double;
begin
x := width-1;
wCoef := height / (maxValue - minValue);
with FValues2BMP do begin
     Width := self.clientWidth;
     height := self.clientHeight;
     canvas.brush.color := FBackgroundColor;
     canvas.fillRect (clientRect);
     end;
with FValuesBMP do begin
     Width := self.clientWidth;
     height := self.clientHeight;
     canvas.brush.color := FBackgroundColor;
     canvas.fillRect (clientRect);
     for i1 := 1 to MAXVALUES do begin
         if x < 0 then break;

         canvas.pen.color := colorPluma (FValues[i1]);
         y := round(FValues[i1] * wCoef) - FMinValue;
         canvas.MoveTo (x, height-1); canvas.lineTo (x, height-1-y);
         if not FTransparent then begin
            canvas.MoveTo (x-1, height-1); canvas.lineTo (x-1, height-1-y);
            end;
         dec (x, 2);
         end;
     end;

end;

procedure THistograph.updateValuesBMP;
var
   wCoef: double;
   x, y: integer;
begin
// Adjust the array of values
for x := MAXVALUES downto 2 do
    FValues[x] := FValues[x-1];
FValues[1] := FValue;

wCoef := height / (maxValue - minValue);
x := width - 1;
with FValues2BMP do begin
     canvas.brush.color := FBackgroundColor;
     canvas.fillRect (clientRect);
     canvas.draw (-2, 0, FValuesBMP); // Slide left the bitmap
     y := round(FValue * wCoef) - FMinValue;
     canvas.pen.color := colorPluma (FValue);
     // And draw only the last value
     canvas.MoveTo (x, height-1); canvas.lineTo (x, height-1-y);
     if not FTransparent then begin
        canvas.MoveTo (x-1, height-1); canvas.lineTo (x-1, height-1-y);
        end;
     end;
FValuesBMP.canvas.draw (0, 0, FValues2BMP);
end;

procedure THistograph.OnTimer (sender: TObject);
begin
updateValuesBMP;
invalidate;
end;

procedure THistograph.paint;
begin
if (FOldWidth <> width) or (FOldHeight <> height) then begin
   FOldWidth := width;
   FOldHeight := height;
   repaintGridBMP;
   repaintValuesBMP;
   end;
FGridBMP.transparent := false;
FValuesBMP.transparent := true;
FValuesBMP.transparentColor := FBackgroundColor;
with FAuxBMP do begin
     canvas.brush.color := FBackgroundColor;
     canvas.fillRect (clientRect);
     canvas.draw (0, 0, FGridBMP);
     canvas.draw (0, 0, FValuesBMP);
     end;

canvas.draw (0, 0, FAuxBMP);
end;

//////////////////////////////////////////////
procedure Register;
begin
  RegisterComponents('Samples', [THistograph]);
end;

end.
